@@ -48,10 +48,22 @@ import cwrapper
 import iso8601
 import dblayer
 
+import error_win
+from ErminigGoogleError import *
+
+import re
+
 currentTime = 0;
 googleEventsTouched = []
 localEventsTouched = []
 
+# pre-compile regexp:
+recur_start = re.compile("DTSTART:(.*)", re.MULTILINE)
+recur_start_fd=re.compile("DTSTART;VALUE=DATE:(.*)", re.MULTILINE)
+recur_end = re.compile("DTEND:(.*)", re.MULTILINE)
+recur_end_fd=re.compile("DTEND;VALUE=DATE:(.*)", re.MULTILINE)
+recur_rrule = re.compile("RRULE:(.*)", re.MULTILINE)
+
 def update_progress(progress, value):
 	if not progress == None:
 		progress.set_fraction(value)
@@ -136,6 +148,39 @@ def removeCancelledEventLocally(cid, gid):
 	dblayer.run("DELETE FROM Xrefs WHERE lid=?", (lid, ))
 	dblayer.commit()
 
+def convert_date(d):
+	if len(d) == 8:
+		return d[:4] + "-" + d[4:6] + "-" + d[6:]
+	else:
+		return d[:4] + "-" + d[4:6] + "-" + d[6:11] + ":" + d[11:13] + \
+			":" + d[13:]
+
+def process_recurrence(rstring):
+	rrule = recur_rrule.search(rstring)
+	if not rrule:
+		return None
+
+	rrule_array = rrule.group(1).split(";")
+	rrule_final_array = []
+	for rule in rrule_array:
+		if not rule.startswith("DTEND") or not rule.startswith("DTSTART"):
+			rrule_final_array.append(rule)
+
+	recurrence = {}
+	recurrence['rrule'] = ";".join(rrule_final_array)
+
+	start = recur_start.search(rstring)
+	end = recur_end.search(rstring)
+
+	if not start:
+		# Full-day event, with recurrence
+		start = recur_start_fd.search(rstring)
+		end = recur_end_fd.search(rstring)
+	
+	recurrence['dtstart'] = convert_date(start.group(1))
+	recurrence['dtend'] = convert_date(end.group(1))
+
+	return recurrence
 	
 def getNewEventsFromGoogle(pid, localSource, remoteSource, lastSync, \
 		                        progress):
@@ -145,7 +190,15 @@ def getNewEventsFromGoogle(pid, localSource, remoteSource, lastSync, \
 	query.updated_max = timestampToIso8601(int(time.time()))
 	query.max_results = "100000"
 
-	feed = google_api.run_google_action(google_api.gd_client.CalendarQuery, query)
+	feed = None
+	try:
+		feed = google_api.run_google_action(google_api.gd_client.CalendarQuery, query)
+	except ErminigGoogleError, e:
+		error_win.display(e.title(), e.description())
+		return
+
+	if not feed:
+		return
 
 	feed_size = len(feed.entry)
 	if feed_size == 0:
@@ -159,21 +212,34 @@ def getNewEventsFromGoogle(pid, localSource, remoteSource, lastSync, \
 		update_progress(progress, current_fraction)
 
 		title = e.title.text
+		rstring = ""
+		start_time = ""
+		end_time = ""
+		full_day = False
 		if e.recurrence <> None:
-			logger.append("Warning: recurring event. Skipping for the moment!")
-			logger.append( "Event was: %s" % (repr(title)))
-			continue
+			recurrence = process_recurrence(e.recurrence.text)
+			if not recurrence:
+				logger.append("Recurrence could not be parsed for")
+				logger.append(title)
+				logger.append("Skipping....")
+				continue
+
+			rstring = recurrence['rrule']
+			start_time = iso8601ToTimestamp(recurrence['dtstart'])
+			end_time = iso8601ToTimestamp(recurrence['dtend'])
+			fullday = isEventFullDay(recurrence['dtstart'])
+		else:
+			start_time = iso8601ToTimestamp(e.when[0].start_time)
+			end_time = iso8601ToTimestamp(e.when[0].end_time)
+			# An event is full-day if it doesn't have any
+			# timing event (only date)
+			fullday = isEventFullDay(e.when[0].start_time)
 
 		where = e.where[0].value_string
 		description =  e.content.text
-		start_time = iso8601ToTimestamp(e.when[0].start_time)
-		end_time = iso8601ToTimestamp(e.when[0].end_time)
 		id = urllib.unquote((e.id.text.rpartition("/"))[2])
 		cdate = time.time()
 
-		# An event is full-day if it doesn't have any
-		# timing event (only date)
-		fullday = isEventFullDay(e.when[0].start_time)
 
 		if (fullday == 1):
 			if ((start_time - end_time) == 0):
@@ -181,7 +247,7 @@ def getNewEventsFromGoogle(pid, localSource, remoteSource, lastSync, \
 
 		event = Event(title, where, description, \
 				start_time, end_time, fullday, id,\
-				cdate)
+				cdate, rrule=rstring)
 		if e.event_status.value == "CANCELED":
 			# remove event locally:
 			removeCancelledEventLocally(localSource, id)
@@ -207,6 +273,33 @@ def queryNewLocalEvents(lastSync, cid):
 
 	return evt_list
 
+def create_rrule(start_time, end_time, rrule):
+	if len(start_time) == 10:
+		# full-day; strip away symbols
+		dt_start = ";VALUE=DATE:" + \
+				filter(lambda c: c not in "-", start_time)
+		dt_end = ";VALUE=DATE:" + \
+				filter(lambda c: c not in "-", end_time)
+	else:
+		# strip away symbols
+		dt_start = ":" + filter(lambda c: c not in "-:", start_time)
+		dt_end = ":" + filter(lambda c: c not in "-:", end_time)
+
+	rrules = rrule.split(";")
+	rrules_t = ""
+	for r in rrules:
+		if r.startswith("UNTIL="):
+			t_pos = r.find("T", 7)
+			rrules_t = rrules_t + r[:t_pos] + ";"
+		else:
+			rrules_t = rrules_t + r + ";"
+
+	rec_data = "DTSTART" + dt_start + "\r\n"
+	rec_data = rec_data + "DTEND" + dt_end + "\r\n"
+	rec_data = rec_data + "RRULE:" + rrules_t.strip(";") + "\r\n"
+
+	return rec_data
+
 def createNewGoogleEvent(evt, googleid, pid):
 
 	start_time = timestampToIso8601(evt.get_start())
@@ -229,9 +322,19 @@ def createNewGoogleEvent(evt, googleid, pid):
 	event.title = atom.Title(text=evt.get_title())
 	event.content = atom.Content(text=evt.get_description())
 	event.where.append(gdata.calendar.Where(value_string=evt.get_where()))
-	event.when.append(gdata.calendar.When(start_time=start_time, end_time=end_time))
+	# Differentiate for recurrence:
+	if evt.get_rrule() <> "":
+		rec_data = create_rrule(start_time, end_time, evt.get_rrule())
+		event.recurrence = gdata.calendar.Recurrence(text=rec_data)
+	else:
+		event.when.append(gdata.calendar.When(start_time=start_time, end_time=end_time))
 
-	new_event = google_api.run_google_action(google_api.gd_client.InsertEvent, event, urllib.quote('/calendar/feeds/' + googleid + '/private/full'))
+	new_event = None
+	try:
+		new_event = google_api.run_google_action(google_api.gd_client.InsertEvent, event, urllib.quote('/calendar/feeds/' + googleid + '/private/full'))
+	except ErminigGoogleError, e:
+		error_win.display(e.title(), e.description())
+		return
 
 	if new_event == None:
 		logger.append("Got a None event....skipping")
@@ -257,7 +360,7 @@ def getNewEventsFromLocal(pid, localSource, remoteSource, lastSync, progress):
 		update_progress(progress, progress_val)
 
 		event = Event(e[3], e[4], e[5], e[1], e[2], e[6], \
-				e[0], 0)
+				e[0], 0, rrule=e[8])
 		event.set_tzOffset(e[7])
 		gid = createNewGoogleEvent(event, remoteSource, pid)
 
@@ -301,7 +404,13 @@ def updateGoogleEvent(evt, googleid, pid):
 	if not gid:
 		return
 
-	event = google_api.run_google_action(google_api.gd_client.GetCalendarEventEntry, urllib.quote("/calendar/feeds/" + googleid + "/private/full/" + gid))
+	event = None
+	try:
+		event = google_api.run_google_action(google_api.gd_client.GetCalendarEventEntry, urllib.quote("/calendar/feeds/" + googleid + "/private/full/" + gid))
+	except ErminigGoogleError, e:
+		error_win.display(e.title(), e.description())
+		return
+
 	if event == None:
 		logger.append("Unable to get event to update!")
 		return
@@ -319,9 +428,18 @@ def updateGoogleEvent(evt, googleid, pid):
 	event.title = atom.Title(text=evt.get_title())
 	event.content = atom.Content(text=evt.get_description())
 	event.where[0] = gdata.calendar.Where(value_string=evt.get_where())
-	event.when[0] = gdata.calendar.When(start_time=start_time, end_time=end_time)
+	# Differentiate for recurrence:
+	if evt.get_rrule() <> "":
+		rec_data = create_rrule(start_time, end_time, evt.get_rrule())
+		event.recurrence = gdata.calendar.Recurrence(text=rec_data)
+	else:
+		event.when[0] = gdata.calendar.When(start_time=start_time, end_time=end_time)
 
-	google_api.run_google_action(google_api.gd_client.UpdateEvent, event.GetEditLink().href, event)
+	try:
+		google_api.run_google_action(google_api.gd_client.UpdateEvent, event.GetEditLink().href, event)
+	except ErminigGoogleError, e:
+		error_win.display(e.title(), e.description())
+		return
 
 def getUpdatedEventsFromLocal(pid, localSource, remoteSource, lastSync, progress):
 	# Those are events to update in Google:
@@ -337,7 +455,7 @@ def getUpdatedEventsFromLocal(pid, localSource, remoteSource, lastSync, progress
 		update_progress(progress, progress_val)
 
 		event = Event(e[3], e[4], e[5], e[1], e[2], e[6], \
-				e[0], 0)
+				e[0], 0, rrule=e[8])
 		event.set_tzOffset(e[7])
 		gid = updateGoogleEvent(event, remoteSource, pid)
 
@@ -357,13 +475,19 @@ def getDeletedEventsFromLocal(pid, localSource, remoteSource, lastSync, progress
 		gid = getGoogleIdFromLocalId(e)
 		if not gid:
 			continue
-		event = google_api.run_google_action(google_api.gd_client.GetCalendarEventEntry, urllib.quote("/calendar/feeds/" + remoteSource + "/private/full/" + gid))
 
-		if event == None:
-			logger.append("Unable to get event to delete!")
-			return 
+		try:
+			event = google_api.run_google_action(google_api.gd_client.GetCalendarEventEntry, urllib.quote("/calendar/feeds/" + remoteSource + "/private/full/" + gid))
 
-		google_api.run_google_action(google_api.gd_client.DeleteEvent, event.GetEditLink().href)
+			if event == None:
+				logger.append("Unable to get event to delete!")
+				return 
+
+			google_api.run_google_action(google_api.gd_client.DeleteEvent, event.GetEditLink().href)
+		except ErminigGoogleError, e:
+			error_win.display(e.title(), e.description())
+			return
+		
 
 def syncFromGoogle(pid, localSource, remoteSource, lastSync, progress=None):
 	getNewEventsFromGoogle(pid, int(localSource), remoteSource, lastSync, \
