@@ -4,9 +4,14 @@ import atom.service
 
 import time
 import urllib
+import socket
 
 import version
 import logger
+import consts
+
+from ErminigGoogleError import *
+from ErminigError import *
 
 gd_client = gdata.calendar.service.CalendarService()
 gd_client.source="Erminig ng %s" % (version.erminig_version)
@@ -27,6 +32,16 @@ def run_google_action(func, *args, **kwargs):
 			if e[0]['status'] == 302:
 				logger.append("** Received spurious redirect - retrying in 2 seconds (attempt %s of %s)" % (i+1, maxattempts))
 				time.sleep(2)
+			elif e[0]['status'] == 401:
+				logger.append("Invalid username or password!")
+				logger.append(repr(e))
+				raise ErminigGoogleError(e)
+				return None
+			elif e[0]['status'] == 403:
+				logger.append("You don't have access to this calendar. Has it been deleted?")
+				logger.append(repr(e))
+				raise ErminigGoogleError(e)
+				return None
 			else:
 				logger.append(repr(e))
 				# XXX Temporary
@@ -41,12 +56,18 @@ def switch_account(credentials):
 	try:
 		gd_client.ProgrammaticLogin()
 	except gdata.service.BadAuthentication, e:
+		logger.append("Invalid username or password!")
 		logger.append(repr(e))
+		raise ErminigError(consts.INVALID_USER_PWD)
 		return False
 
 	except gdata.service.CaptchaRequired, e:
 		logger.append(repr(e))
 		return False
+	except socket.gaierror, e:
+		logger.append("Unable to connect! Check internet connection")
+		raise ErminigError(consts.NO_INET_CONNECTION)
+		return False
 
 	return True
 
