@@ -4,10 +4,14 @@ import gtk
 import consts
 import fremantle
 import google_accounts
+import settings_ui
 import google_api
 import profiles
 import logger
 
+from ErminigError import *
+import error_win
+
 def display_google_auth_error(win):
 	note = hildon.hildon_note_new_information(win, \
 		"Could not connect to your Google account. Check username "\
@@ -105,8 +109,10 @@ def add_profile(widget, data):
 
 	data.destroy()
 
-def new_remote_source(widget, data):
+def new_remote_source(widget, hint):
 	global current_google_account_selected
+	global local_selector
+
 	dialog = gtk.Dialog()
 	dialog.set_title("New Google Calendar")
 
@@ -115,6 +121,11 @@ def new_remote_source(widget, data):
 	label = gtk.Label("Please enter the name of a new Google calendar")
 	new_cal_entry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
 	new_cal_entry.set_placeholder("Calendar name")
+	hint = local_selector.get_current_text()
+	if not hint:
+		hint = ""
+	new_cal_entry.set_text(hint)
+	new_cal_entry.select_region(0, len(hint))
 
 	dialog.vbox.add(label)
 	dialog.vbox.add(new_cal_entry)
@@ -133,7 +144,9 @@ def new_remote_source(widget, data):
 		
 	dialog.destroy()
 
-def new_local_source(widget, data):
+def new_local_source(widget, hint):
+	global remote_selector
+
 	dialog = gtk.Dialog()
 	dialog.set_title("New Local Calendar")
 
@@ -142,6 +155,11 @@ def new_local_source(widget, data):
 	label = gtk.Label("Please enter the name of a new local calendar")
 	new_cal_entry = hildon.Entry(gtk.HILDON_SIZE_AUTO)
 	new_cal_entry.set_placeholder("Calendar name")
+	hint = remote_selector.get_current_text()
+	if not hint:
+		hint = ""
+	new_cal_entry.set_text(hint)
+	new_cal_entry.select_region(0, len(hint))
 
 	dialog.vbox.add(label)
 	dialog.vbox.add(new_cal_entry)
@@ -221,6 +239,7 @@ def update_local_data_source(source_id):
 def create_local_selector(win):
 	global local_source_list
 	global local_selector
+
 	local_selector = hildon.TouchSelector()
 	local_source_list = gtk.ListStore(int, str)
 
@@ -238,7 +257,7 @@ def create_local_selector(win):
 			gtk.HILDON_SIZE_FINGER_HEIGHT, \
 			hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
 	new_btn.set_label("New...")
-	new_btn.connect("clicked", new_local_source, win)
+	new_btn.connect("clicked", new_local_source, "hint")
 
 	hbox = gtk.HBox()
 	hbox.pack_start(picker)
@@ -252,13 +271,17 @@ def update_remote_data_source(sync_type, google_acct_id):
 
 	if sync_type == consts.SYNC_TYPE_CAL:
 		hildon.hildon_gtk_window_set_progress_indicator(dialog, 1)
-		if not google_api.switch_account(\
-				google_accounts.get_account_by_id(\
-				google_acct_id)):
-			display_google_auth_error(dialog)
-			google_new_btn.set_sensitive(False)
-			google_item_picker.set_sensitive(False)
-			hildon.hildon_gtk_window_set_progress_indicator(dialog, 0)
+		try:
+			if not google_api.switch_account(\
+					google_accounts.get_account_by_id(\
+					google_acct_id)):
+				display_google_auth_error(dialog)
+				google_new_btn.set_sensitive(False)
+				google_item_picker.set_sensitive(False)
+				hildon.hildon_gtk_window_set_progress_indicator(dialog, 0)
+				return
+		except ErminigError, e:
+			error_win.display(e.title(), e.description())
 			return
 
 		google_new_btn.set_sensitive(True)
@@ -291,7 +314,7 @@ def create_remote_selector(win):
 			gtk.HILDON_SIZE_FINGER_HEIGHT, \
 			hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
 	google_new_btn.set_label("New...")
-	google_new_btn.connect("clicked", new_remote_source, win)
+	google_new_btn.connect("clicked", new_remote_source, "hint")
 	google_new_btn.set_sensitive(False)
 
 	hbox = gtk.HBox()
@@ -360,6 +383,7 @@ def account_selector_changed(selector, user_data):
 
 def create_account_selector():
 	global account_selector
+	global account_list
 
 	account_selector = hildon.TouchSelector()
 
@@ -479,11 +503,25 @@ def fill_form(pid):
 			break
 		iter = model.iter_next(iter)
 
+def ask_for_google_account_creation(parent):
+	dialog = gtk.MessageDialog(parent=parent, flags=0, \
+			type=gtk.MESSAGE_INFO, buttons=gtk.BUTTONS_NONE)
+	dialog.set_title("No Google Account defined")
+	dialog.format_secondary_text("No Google account has been defined yet.\n You will now have the possibility to add an account.")
+	dialog.add_button(gtk.STOCK_OK, gtk.RESPONSE_OK)
+	dialog.run()
+	dialog.destroy()
+	settings_ui.add_google_acct_dialog(parent, None, external=True)
+
+	for id, acct in google_accounts.get_registered_accounts():
+		account_list.append([int(id), acct])
 
 def display(win, pid = None):
 	global dialog
 	global editing
 	global current_profile_id
+	global account_selector
+	
 	dialog = gtk.Dialog()
 	dialog.set_title("Add new Profile")
 
@@ -500,6 +538,13 @@ def display(win, pid = None):
 		editing = True
 	
 	dialog.show_all()
+	if google_accounts.get_registered_accounts_count() == 0:
+		ask_for_google_account_creation(dialog)
+
+	if google_accounts.get_registered_accounts_count() == 1:
+		# Pre-select the only Google account available:
+		account_selector.set_active(0, 0)
+
 	dialog.run()
 
 local_source_list = None
@@ -507,6 +552,7 @@ local_selector = None
 remote_selector = None
 remote_sources = None
 google_new_btn = None
+account_list = None
 google_item_picker = None
 account_selector = None
 direction_selector = None
